//
// Controle de frequencia da Jornada Academica da FACIN/PUCRS
// Porto Alegre, 2, 3 e 4 de setembro de 2008. 
//
// Autor: Eduardo Augusto Bezerra
// Data: 25 de agosto de 2008.
//
// Realiza a leitura de um codigo de barras pela porta serial
// e grava o dado lido em um arquivo texto, juntamente com
// a data e hora.
//
// - O arquivo de saida e' aberto no modo append, e novos dados
//   lidos sao sempre adicionados no final do arquivo.
//
// - Cada linha do arquivo possui apenas um registro composto por
//   data, hora e dado lido da leitora de codigo de barras.
//
// - Formato dos registros do arquivo:  DD-MM-AA HH:MM:SS Matricula
//   onde: DD = dia, MM = mes, AA = ano, HH = hora, MM = minuto, SS = segundo
//   Matricula = codigo de barras recebido pela serial
//
// - A leitora de codigo de barras realiza a leitura e pre-processamento dos
//   codigos de barras, e envia os caracteres, um byte de cada vez, ja' no formato
//   ASCII.
//
// - A leitora esta' pre-configurada para comunicacao serial com 8 bits de 
//   dados, 1 stop bit, sem paridade, 9600 bps
//
// Para compilar o programa e gerar o executavel, usar make (ver o arquivo Makefile
// para as regras de compilacao.
//
//
// Log de alteracoes:
//
// 25 ago 2008 - Criacao do programa
// 27 ago 2008 - Inclusao de GUI no programa - FLTK
//

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Output.H>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>

#include "BarCodeReader.c"

int main(int argc, char **argv) {
   char tecla;
   int i=0, result;
   char matricula[12];
   time_t     now;
   struct tm  ts;
   char       buf[80], buf1[80];

   FILE *fp;

   if (setuid(0) == 0)
      printf ("Processo agora eh root\n");

   openSerial(1);

   fp=fopen ("../freq_2008_2.txt","a+");

   if (!fp) {
        printf ("Erro na abertura do arquivo.");
        return 0;
   }


   Fl_Window *window = new Fl_Window(200, 200, 300,180);
   Fl_Box *box = new Fl_Box(FL_NO_BOX,20,40,260,100,"Jornada Academica 2008");
   box->labelfont(FL_BOLD+FL_ITALIC);
   box->labelsize(36);
   box->labeltype(FL_SHADOW_LABEL);
   //window->fullscreen();
   window->clear_border();
   //window->set_modal();
   window->end();
   window->show(argc, argv);
   Fl::wait(0);

   fcntl(fd, F_SETFL, 0);
   printf("\nFavor passar o cartao na leitora de codigo de barras: \n");

   //for(;;) {

      i = 0;
      memset(matricula,0, sizeof(matricula));
      while(read(fd, &tecla, 1) > 0){    // leitura de um byte por vez da serial
         matricula[i] = tecla;
         if (tecla == '\n'){                   // os crachas da PUC possuem 10 caracteres
            i = 0;
            time(&now);
            ts = *localtime(&now);
            // Formata e imprime o tempo, "ddd yyyy-mm-dd hh:mm:ss zzz"
            // strftime(buf, sizeof(buf), "%a %Y-%m-%d %H:%M:%S %Z", &ts);
            memset(buf,0, sizeof(buf));
            strftime(buf, sizeof(buf), "%d-%m-%Y %H:%M:%S", &ts);
            printf("=>%s %s", buf, matricula);
            sprintf(buf1,"Matricula\n%s", matricula);
            box->label(buf1);
            box->redraw();
            Fl::wait(0);
            result = fprintf(fp, "=>%s %s", buf, matricula);
            fflush(fp);
            memset(matricula,0, sizeof(matricula));
            printf("\nFavor passar o cartao na leitora de codigo de barras: \n");
         } else
            i++;
      }
      printf("saiu\n\n");
      tecla=0;

   //}
   close (fd);
   fclose (fp);
   return 0;

}


